#include <QtWaylandClient/private/qwaylandscreen_p.h>

#include "DropDownSurface.hpp"
#include "DropDownSurfaceImpl.hpp"

DesQ::DropDownShell::DropDownShell( QtWaylandDisplay *display, wlrLayerShell *shell ) : wlrLayerShell( shell->object() ) {

    m_display = display;
};

DesQ::DropDownShell::~DropDownShell() {

    zwlr_layer_shell_v1_destroy( object() );
};

DesQ::DropDownSurface* DesQ::DropDownShell::createLayerSurface( QtWaylandWindow *window ) {

    return new DesQ::DropDownSurface( this, window );
};

wlrLayerSurface create_layer_surface( DesQ::DropDownShell *shell, QtWaylandWindow *window ) {

    return shell->get_layer_surface( window->wlSurface(), window->waylandScreen()->output(), wlrLayerShell::layer_top, "qt" );
};

DesQ::DropDownSurface::DropDownSurface( DesQ::DropDownShell *shell, QtWaylandWindow *window ) :
		QtWaylandShellSurface( window ),
		wlrLayerSurface( create_layer_surface( shell, window ) ) {

	m_window = window;

    set_size( m_window->window()->width(), m_window->window()->height() );
	apply();
};

void DesQ::DropDownSurface::apply() {

    set_anchor( m_anchor );
    set_exclusive_zone( m_exclusive_zone );
    set_margin( m_margin.top(), m_margin.right(), m_margin.bottom(), m_margin.left() );

    set_keyboard_interactivity( m_keyboard_interactivity );

    // Commit and roundtrip immediately
    m_window->commit();
    m_window->display()->forceRoundTrip();
};

void DesQ::DropDownSurface::setAnchor( uint32_t anchor ) {

    m_anchor = anchor;
    apply();
};

void DesQ::DropDownSurface::setExclusiveZone( int32_t exclusive_zone ) {

    m_exclusive_zone = exclusive_zone;
    apply();
};

void DesQ::DropDownSurface::setSize( uint32_t width, uint32_t height ) {

    m_width = width;
    m_height = height;

    set_size( m_width, m_height );
    apply();
};

void DesQ::DropDownSurface::setMargin( int32_t top, int32_t right, int32_t bottom, int32_t left ) {

	m_margin = QMargins( left, top, right, bottom );
    apply();
};

void DesQ::DropDownSurface::setKeyboardInteractivity( bool keyboard_interactivity ) {

    m_keyboard_interactivity = keyboard_interactivity;
    apply();
};

DesQ::DropDownSurface::~DropDownSurface() {

    destroy();
};

void DesQ::DropDownSurface::zwlr_layer_surface_v1_closed() {

    m_window->window()->close();
};

void DesQ::DropDownSurface::applyConfigure() {

    m_window->resizeFromApplyConfigure( m_pendingSize );
};

void DesQ::DropDownSurface::zwlr_layer_surface_v1_configure( uint32_t serial, uint32_t width, uint32_t height ) {

    m_pendingSize = QSize( width, height );//{width, height};
    m_window->applyConfigureWhenPossible();
    ack_configure( serial );
    is_configured = true;
};

DesQ::DropDownSurface* DesQ::DropDownSurface::fromQWindow( QWindow *window ) {

    QtWaylandClient::QWaylandWindow *wWindow = dynamic_cast<QtWaylandClient::QWaylandWindow *>( window->handle() );
    if ( not wWindow )
        return nullptr;

    return dynamic_cast<DesQ::DropDownSurface *>( wWindow->shellSurface() );
};
