#pragma once

#include <wayland-client.h>

#include <QtWaylandClient/private/qwaylandshellsurface_p.h>
#include <QtWaylandClient/private/qwaylandshellintegration_p.h>
#include <QtWaylandClient/private/qwaylanddisplay_p.h>
#include <QtWaylandClient/private/qwaylandwindow_p.h>

#include <qwayland-wlr-layer-shell-unstable-v1.h>

typedef QtWaylandClient::QWaylandShellSurface QtWaylandShellSurface;
typedef QtWaylandClient::QWaylandDisplay QtWaylandDisplay;
typedef QtWaylandClient::QWaylandWindow QtWaylandWindow;

typedef QtWayland::zwlr_layer_shell_v1 wlrLayerShell;
typedef QtWayland::zwlr_layer_surface_v1 wlrLayerSurface;

namespace DesQ {
    class DropDownShell;
    class DropDownSurface;
};

class DesQ::DropDownSurface : public QtWaylandShellSurface, public wlrLayerSurface {
    Q_OBJECT

    public:
        DropDownSurface( DesQ::DropDownShell *shell, QtWaylandWindow *window );
        virtual ~DropDownSurface();

        void setAnchor( uint32_t anchor );
        void setExclusiveZone( int32_t zone );
        void setMargin( int32_t top, int32_t right, int32_t bottom, int32_t left );
        void setSize( uint32_t width, uint32_t height );
        void setKeyboardInteractivity( bool interactive );
        void applyConfigure() override;

        void apply();

        static DesQ::DropDownSurface* fromQWindow( QWindow *window );

    protected:
        void zwlr_layer_surface_v1_configure( uint32_t serial, uint32_t width, uint32_t height ) override;
        void zwlr_layer_surface_v1_closed() override;

    private:
        QtWaylandWindow *m_window;
        bool is_configured = false;

        QSize m_pendingSize = {0, 0};

        uint32_t m_width = 0;
        uint32_t m_height = 0;
        uint32_t m_layer = 0;
        uint32_t m_anchor = 0;
        int32_t m_exclusive_zone = 0;
        bool m_keyboard_interactivity = false;
        QMargins m_margin;
};
