/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QTimer>

class QWindow;
class QScreen;
class LayerSurface;
class DesQHotSpot;
class DesQHotSpotPrivate;

struct zwf_shell_manager_v2;
struct zwf_hotspot_v2;

class DesQAutoHideWindow : public QObject {
	Q_OBJECT

	public:
		DesQAutoHideWindow( QWindow *window, zwf_shell_manager_v2 *wayfireShell, QObject *parent = nullptr );
		void initialize( LayerSurface *lSurface );

	private:
		QWindow *m_window;
		zwf_shell_manager_v2 *m_wayfireShell;
		DesQHotSpot *m_edgeHotspot;
		DesQHotSpot *m_dockHotspot;
		LayerSurface *m_layerSurface;

		int m_dockPos;
		uint m_edgeHeight;
		uint m_showDuration;
		uint m_hideDuration;

		bool m_isEdgeHotspot;

		QTimer *m_hideTimer;

		void createEdgeHS();
		void createDockHS();
		void updateMargin();
};

class DesQHotSpot : public QObject {
	Q_OBJECT

	public:
		enum Position {
			Top    = 1,
			Bottom = 2,
			Left   = 4,
			Right  = 8,
			TopLeft     = Top | Left,
			TopRight    = Top | Right,
			BottomLeft  = Bottom | Left,
			BottomRight = Bottom | Right
		};

		Q_ENUM( Position )

		DesQHotSpot( QScreen *screen, zwf_shell_manager_v2 *wayfireShell, QObject *parent = nullptr );
		void createHotspot( DesQHotSpot::Position pos, uint height, uint duration );
		void destroy();

	signals:
		void enteredHotspot();
		void leftHotspot();

	protected:
		void zwf_hotspot_v2_enter();
		void zwf_hotspot_v2_leave();

		Q_DECLARE_PRIVATE( DesQHotSpot );
		DesQHotSpotPrivate *d_ptr;
};
