/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QMargins>
#include <QSize>

class QWindow;
class LayerSurface;

struct zwlr_layer_shell_v1;
struct zwlr_layer_surface_v1;

class LayerShellPrivate;
class LayerSurfacePrivate;

class LayerShell : public QObject {

	Q_OBJECT

	public:
		enum LayerType {
			Background = 0,
			Bottom = 1,
			Top = 2,
			Overlay = 3
		};

		LayerShell( struct ::zwlr_layer_shell_v1 *lShell, QObject *parent = nullptr );
		struct ::zwlr_layer_surface_v1 *getLayerSurface( QWindow *window, LayerShell::LayerType layer, const QString &layer_namespace );

	protected:
		Q_DECLARE_PRIVATE( LayerShell );
		LayerShellPrivate *d_ptr;
};

class LayerSurface : public QObject {
	Q_OBJECT

	public:
		enum SurfaceAnchor {
			None = 0,
			Top = (1 << 0),
			Bottom = (1 << 1),
			Left = (1 << 2),
			Right = (1 << 3)
		};
		Q_DECLARE_FLAGS(SurfaceAnchors, SurfaceAnchor)

		LayerSurface( QWindow *window, QObject *parent = nullptr );
		void setup( LayerShell *lShell, LayerShell::LayerType type );
		void apply();

		void setSurfaceSize( const QSize &surfaceSize );
		void setAnchors( const SurfaceAnchors &anchors );
		void setExclusiveZone( int exclusiveZone );
		void setMargins( const QMargins &margins );
		void setKeyboardInteractivity( bool keyboardInteractivity );

	signals:
		void configured( QSize size );

	protected:
		void zwlr_layer_surface_v1_configure( uint32_t serial, uint32_t width, uint32_t height );
		void zwlr_layer_surface_v1_closed();

		Q_DECLARE_PRIVATE( LayerSurface );
		LayerSurfacePrivate *d_ptr;

	private:
		QWindow *mWindow;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(LayerSurface::SurfaceAnchors)
