/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>

class WindowHandle;
class WindowHandlePrivate;
class WindowManagerPrivate;

struct wl_seat;
struct wl_array;
struct wl_output;
struct zwlr_foreign_toplevel_handle_v1;
struct zwlr_foreign_toplevel_manager_v1;

typedef QList<WindowHandle*> WindowHandles;

class WindowManager : public QObject {
	Q_OBJECT

	public:
		explicit WindowManager(struct ::zwlr_foreign_toplevel_manager_v1 *tlManager, struct wl_seat *seat, QObject *parent = nullptr);

		WindowHandles windowHandles() const;
		void activateTask( WindowHandle *handle );

	signals:
		void newWindowHandle( WindowHandle *tlHandle );

	protected:
		void zwlr_foreign_toplevel_manager_v1_toplevel( struct ::zwlr_foreign_toplevel_handle_v1 *handle );
		void zwlr_foreign_toplevel_manager_v1_finished();

		Q_DECLARE_PRIVATE( WindowManager )
		WindowManagerPrivate *d_ptr;

	private:
		void closeHandle( WindowHandle *tlHandle );
};

class WindowHandle : public QObject {
	Q_OBJECT

	public:
		enum WindowState {
			None = (1 << 0),
			Activated = (1 << 1),
			Maximized = (1 << 2),
			Minimized = (1 << 3)
		};

		Q_DECLARE_FLAGS(WindowStates, WindowState)

		WindowHandle( struct ::zwlr_foreign_toplevel_handle_v1 *tlHandle, QObject *parent = nullptr );

		QString appID() const;
		QString appTitle() const;

		void set_maximized();
        void unset_maximized();
        void set_minimized();
        void unset_minimized();
		void activate( struct ::wl_seat *seat );
        void close();

		WindowHandle::WindowStates appStates() const;

	signals:
		void handleUpdated();
		void handleClosed( WindowHandle *tlHandle );

	protected:
		void zwlr_foreign_toplevel_handle_v1_title(const QString &title);
		void zwlr_foreign_toplevel_handle_v1_app_id(const QString &app_id);
		void zwlr_foreign_toplevel_handle_v1_output_enter(struct ::wl_output *output);
		void zwlr_foreign_toplevel_handle_v1_output_leave(struct ::wl_output *output);
		void zwlr_foreign_toplevel_handle_v1_state(wl_array *state);
		void zwlr_foreign_toplevel_handle_v1_done();
		void zwlr_foreign_toplevel_handle_v1_closed();

		Q_DECLARE_PRIVATE( WindowHandle )
		WindowHandlePrivate *d_ptr;
};

Q_DECLARE_OPERATORS_FOR_FLAGS( WindowHandle::WindowStates )
