/*
	*
	* This file is a part of IonShell.
	* IonShell is the Desktop Shell App for Ion DE
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>

#include "qwayland-wlr-foreign-toplevel-management-unstable-v1.h"

#include "WindowManager.hpp"

class WindowHandlePrivate;
class WindowManagerPrivate;

struct zwlr_foreign_toplevel_manager_v1;
struct zwlr_foreign_toplevel_handle_v1;

class WindowManagerPrivate : public QtWayland::zwlr_foreign_toplevel_manager_v1 {

	public:
		WindowManagerPrivate( struct ::zwlr_foreign_toplevel_manager_v1 *tlManager, struct wl_seat *seat, WindowManager *ptr );

		WindowHandles windowHandles() const;
		void activateTask( WindowHandle *handle );

	protected:
		void zwlr_foreign_toplevel_manager_v1_toplevel(struct ::zwlr_foreign_toplevel_handle_v1 *handle) override;
		void zwlr_foreign_toplevel_manager_v1_finished() override;

		Q_DISABLE_COPY( WindowManagerPrivate )
		Q_DECLARE_PUBLIC( WindowManager )
		WindowManager *q_ptr;

	private:
		WindowHandles m_tlHandleList;
		struct wl_seat *m_seat;
};

class WindowHandlePrivate : public QtWayland::zwlr_foreign_toplevel_handle_v1 {

	public:
		WindowHandlePrivate( struct ::zwlr_foreign_toplevel_handle_v1 *tlHandle, WindowHandle *ptr );

	protected:
		void zwlr_foreign_toplevel_handle_v1_title(const QString &title) override;
		void zwlr_foreign_toplevel_handle_v1_app_id(const QString &app_id) override;
		void zwlr_foreign_toplevel_handle_v1_output_enter(struct ::wl_output *output) override;
		void zwlr_foreign_toplevel_handle_v1_output_leave(struct ::wl_output *output) override;
		void zwlr_foreign_toplevel_handle_v1_state(wl_array *state) override;
		void zwlr_foreign_toplevel_handle_v1_done() override;
		void zwlr_foreign_toplevel_handle_v1_closed() override;

		Q_DISABLE_COPY( WindowHandlePrivate )
		Q_DECLARE_PUBLIC( WindowHandle )
		WindowHandle *q_ptr;

	private:
		QString m_appID;
		QString m_appTitle;
		WindowHandle::WindowStates m_appStates;
};
